@@ -162,7 +162,9 @@ class Grouper:
         >>> np.add.reduceat(df["b"], df.groupby("a").reduceat_indexes())
         array([ 9, 13, 27])
         """
-        indexes = np.empty(len(self.counts), dtype=int)
+        if (size := len(self.counts)) == 0:
+            return np.array([], dtype=int)
+        indexes = np.empty(size, dtype=int)
         indexes[0] = 0
         np.cumsum(self.counts[:-1], out=indexes[1:])
         return indexes
@@ -178,7 +180,9 @@ class Grouper:
         >>> df["a"][df.groupby("a").group_indexes()]
         array([1, 2, 3])
         """
-        indexes = np.empty(len(self.counts), dtype=int)
+        if (size := len(self.counts)) == 0:
+            return np.array([], dtype=int)
+        indexes = np.empty(size, dtype=int)
         indexes[0] = 0
         np.cumsum(self.counts[:-1], out=indexes[1:])
         return self.order[indexes]
@@ -334,7 +338,7 @@ class DataFrame(metaclass=PureStaticDataFrameMethods):
             return DataFrame({k: columns[k] for k in item}, index=self._index)
         return self._columns[item]
 
-    def __setitem__(self, key: Hashable, value: npt.ArrayLike) -> None:
+    def __setitem__(self, key: Hashable, value: Any) -> None:
         """
         Add or replace a column.
 
@@ -342,7 +346,10 @@ class DataFrame(metaclass=PureStaticDataFrameMethods):
         :param value: Numpy array with column values. We set without copying, so the user should \
                       provide an array copy as needed.
         """
-        value = np.atleast_1d(np.squeeze(np.asarray(value)))
+        if np.isscalar(value):
+            value = np.full(len(self), value)
+        else:
+            value = np.atleast_1d(np.squeeze(np.asarray(value)))
         if value.ndim != 1:
             raise ValueError(f"numpy array must be one-dimensional, got shape {value.shape}")
         if len(self) != len(value) and self._columns:
@@ -1106,7 +1113,11 @@ class DataFrame(metaclass=PureStaticDataFrameMethods):
         mapped_bys = []
         for c in by:
             unique_values, inverse_indexes = np.unique(c, return_inverse=True)
-            if na_position == "first" and unique_values[-1] != unique_values[-1]:
+            if (
+                na_position == "first"
+                and len(unique_values)
+                and unique_values[-1] != unique_values[-1]
+            ):
                 inverse_indexes += 1
                 inverse_indexes[inverse_indexes == len(unique_values)] = 0
             mapped_bys.append(inverse_indexes)
@@ -1120,7 +1131,7 @@ class DataFrame(metaclass=PureStaticDataFrameMethods):
     ):
         if subset is None:
             subset = self._columns.keys()
-        elif subset in self._columns:
+        elif isinstance(subset, Hashable) and subset in self._columns:
             subset = (subset,)
         if not isinstance(subset, Iterable):
             raise TypeError(f"subset of columns is not recognized: {type(subset)}")
